/*
 * Celestial Fusion
 * Software for digitized engine data acquisition and processing
 *
 * Author: James McCrae
 */

#include <allegro.h>
#include <string.h>
#include <stdio.h>
#include <math.h>

#include "globals.h"
#include "poll.h"
#include "parport.h"
#include "simulate.h"
#include "gui.h"

bool DEBUGMODE;

int outport;
int inport;

int NUMINPUTSENSORS;

int TACHBUSADDR;
int SPEEDBUSADDR;

int GAUGERESOLUTIONX;
int GAUGERESOLUTIONY;

int GRAPHXLINES;
int GRAPHYLINES;

int GRAPHDATASIZE;

double POWEROVERTIMECURVEDURATION_SEC;
double POWEROVERTIMECURVERANGE_HP;

double ACCELERATIONOVERTIMEDURATION_SEC;
double ACCELERATIONOVERTIMERANGE_KMHSEC;

double VELOCITYOVERTIMEDURATION_SEC;

double MAXTACHOMETERRANGE;
double TACHOMETERREDLINE;
double MAXSPEEDOMETERRANGE;
double SPEEDPULSESPERKM;

double VEHICLEWEIGHTKG;

int RESOLUTIONX;
int RESOLUTIONY;

float PULSETIMECONSIDERED;

float POLLDELAYSEC;

float UPDATESPERSEC;

double CALC_H_APPROX;
int AVGSMOOTHSIZE;
double ERRORCORRECTIONDECELERATIONTIME_SEC;

/* Unconfigurable Global variables */
double TRIPOMETER=0.0;
int NUMINPUTSAMPLES;
int GRAPHXSIZEPIXELS;
int GRAPHYSIZEPIXELS;

/* Function declarations */
void alleg_init();
void cmdLineError();
void loadConfig(char *filename);

/* Main Method */
int main(int argc, char ** argv) {
  
  bool dosimulate=false;
  DEBUGMODE=false;

  loadConfig("config.cfg");

  //parse command line arguments
  for (int i=1;i<argc;i++) {
    if (strcasecmp(argv[i],"-S")==0)
      dosimulate=true;
    else if (strcasecmp(argv[i],"-DEBUG")==0) {
      dosimulate=true;
      DEBUGMODE=true;
    }
    else if (strcasecmp(argv[i],"-R")==0) {
      if (argc<=i+2) cmdLineError();
      RESOLUTIONX=atoi(argv[i+1]);
      RESOLUTIONY=atoi(argv[i+2]);
      i+=2;
    }
    else if (strcasecmp(argv[i],"-T")==0) {
      if (argc<=i+1) cmdLineError();
      PULSETIMECONSIDERED=atof(argv[i+1]);
      i+=1;
    }
    else if (strcasecmp(argv[i],"-D")==0) {
      if (argc<=i+1) cmdLineError();
      POLLDELAYSEC=atof(argv[i+1]);
      i+=1;
    }
    else if (strcasecmp(argv[i],"-U")==0) {
      if (argc<=i+1) cmdLineError();
      UPDATESPERSEC=atof(argv[i+1]);
      i+=1;
    }
    else
      cmdLineError();
  }

  //dynamically set the number of input samples to the time for
  //important polls as well as the time between each poll
  NUMINPUTSAMPLES=(int)(ceil(PULSETIMECONSIDERED/POLLDELAYSEC));
  
  //set up parallel port IO
  
  ParallelPortIO *ppio=new ParallelPortIO();
  ppio->setSimulated(dosimulate);

  //set up simulator (used if needed)
  Simulator sim;

  //set up environment
  alleg_init();
  //set up GUI
  GUI g;

  clock_t lastpolltime=0;

  do {
    
    if (ppio->getSimulated()) {
      sim.update(ppio);      
    }
    else {      
      ppio->addPoll(TACHBUSADDR,((double)clock())/CLOCKS_PER_SEC,ppio->pollSensor(TACHBUSADDR));
      ppio->addPoll(SPEEDBUSADDR,((double)clock())/CLOCKS_PER_SEC,ppio->pollSensor(SPEEDBUSADDR));
    }

    //update tripometer
    TRIPOMETER+=(ppio->obtainLastPoll(SPEEDBUSADDR).getReading())/((double)SPEEDPULSESPERKM);     
 
    g.incrementPoll();
    g.update(ppio,&sim);            
 
    //delay
    //delay(5000);
    while (clock()-lastpolltime<CLOCKS_PER_SEC*POLLDELAYSEC) { }
    lastpolltime=clock();
    
  } while (true); 

}

/*
 * Sets up keyboard, timer and graphics mode.
 */
void alleg_init() {
  if (allegro_init()!=0) {
    exit(1);
  }  
  install_keyboard();

  set_color_depth(8);
  set_gfx_mode(GFX_AUTODETECT,RESOLUTIONX,RESOLUTIONY,0,0); 
  text_mode(-1);

  remove_timer();
}


void cmdLineError() {

  printf("Invalid usage of CF!\n");
  printf("Proper command line usage is of the form:\n");
  printf("  cf.exe [-S] [-R X Y] [-T t] [-D t] [-U n]\n");
  printf("-S\tSimulate sensor inputs (default is to not simulate)\n");
  printf("-R\tResolution of graphics, where X and Y are pixels (default 640 480)\n");
  printf("-T\tTime of polls taken t seconds ago considered (default is 1.0)\n");
  printf("-D\tDelay t seconds between polls (default is 0.05)\n");
  printf("-U\tUpdate GUI n times per second (default is 10)\n");

  exit(-1);

}

void loadConfig(char *filename) {

  FILE *configfile=fopen(filename,"r");
  char *inputline=new char[80];
  char *variable=new char[60];
  char *value=new char[20];  

  do {

    fgets(inputline,80,configfile);

    if (inputline[0]!='/'&&strlen(inputline)>2) {
      strncpy(variable,inputline,(strchr(inputline,'=')-inputline));
      variable[strchr(inputline,'=')-inputline]='\0';
      strcpy(value,strchr(inputline,'=')+1);
    
      if (strcmp(variable,"OUTPORT")==0)
        outport=atoi(value);
      else if (strcmp(variable,"INPORT")==0)
        inport=atoi(value);
      else if (strcmp(variable,"NUMINPUTSENSORS")==0)
        NUMINPUTSENSORS=atoi(value);
      else if (strcmp(variable,"TACHBUSADDR")==0)
        TACHBUSADDR=atoi(value);
      else if (strcmp(variable,"SPEEDBUSADDR")==0)
        SPEEDBUSADDR=atoi(value);
      else if (strcmp(variable,"PULSETIMECONSIDERED")==0)
        PULSETIMECONSIDERED=atof(value);
      else if (strcmp(variable,"POLLDELAYSEC")==0)
        POLLDELAYSEC=atof(value);
      else if (strcmp(variable,"CALC_H_APPROX")==0)
        CALC_H_APPROX=atof(value);
      else if (strcmp(variable,"AVGSMOOTHSIZE")==0)
        AVGSMOOTHSIZE=atoi(value);
      else if (strcmp(variable,"MAXTACHOMETERRANGE")==0)
        MAXTACHOMETERRANGE=atof(value);
      else if (strcmp(variable,"TACHOMETERREDLINE")==0)
        TACHOMETERREDLINE=atof(value);
      else if (strcmp(variable,"MAXSPEEDOMETERRANGE")==0)
        MAXSPEEDOMETERRANGE=atof(value);
      else if (strcmp(variable,"SPEEDPULSESPERKM")==0)
        SPEEDPULSESPERKM=atof(value);
      else if (strcmp(variable,"VEHICLEWEIGHTKG")==0)
        VEHICLEWEIGHTKG=atof(value);
      else if (strcmp(variable,"RESOLUTIONX")==0)
        RESOLUTIONX=atoi(value);
      else if (strcmp(variable,"RESOLUTIONY")==0)
        RESOLUTIONY=atoi(value);
      else if (strcmp(variable,"GAUGERESOLUTIONX")==0)
        GAUGERESOLUTIONX=atoi(value);
      else if (strcmp(variable,"GAUGERESOLUTIONY")==0)
        GAUGERESOLUTIONY=atoi(value);
      else if (strcmp(variable,"GRAPHXLINES")==0)
        GRAPHXLINES=atoi(value);
      else if (strcmp(variable,"GRAPHYLINES")==0)
        GRAPHYLINES=atoi(value);
      else if (strcmp(variable,"GRAPHDATASIZE")==0)
        GRAPHDATASIZE=atoi(value);
      else if (strcmp(variable,"UPDATESPERSEC")==0)
        UPDATESPERSEC=atof(value);
      else if (strcmp(variable,"POWEROVERTIMECURVEDURATION_SEC")==0)
        POWEROVERTIMECURVEDURATION_SEC=atof(value);
      else if (strcmp(variable,"POWEROVERTIMECURVERANGE_HP")==0)
        POWEROVERTIMECURVERANGE_HP=atof(value);
      else if (strcmp(variable,"ACCELERATIONOVERTIMEDURATION_SEC")==0)
        ACCELERATIONOVERTIMEDURATION_SEC=atof(value);
      else if (strcmp(variable,"ACCELERATIONOVERTIMERANGE_KMHSEC")==0)
        ACCELERATIONOVERTIMERANGE_KMHSEC=atof(value);
      else if (strcmp(variable,"VELOCITYOVERTIMEDURATION_SEC")==0)
        VELOCITYOVERTIMEDURATION_SEC=atof(value);
      else if (strcmp(variable,"ERRORCORRECTIONDECELERATIONTIME_SEC")==0)
        ERRORCORRECTIONDECELERATIONTIME_SEC=atof(value);
    }

  } while (!feof(configfile));

  fclose(configfile);

}


